import java.awt.*;
import java.util.*;

/**
    A snake game applet -- the snake class
    @author Vincent Wing Hei, Ho
    @version 02.05.2000
*/
public class Snake {
	
    /** Current X position of snake */
    private int snakeX;
    /** Current Y position of snake */
    private int snakeY;
    /** Vector for storing positions of snake tail */
    private Vector snakeTail;
    /** Current direction of the snake */
    private int direction;
    /** To store if the snake is dead */
    private boolean isDead;
    
    /** Symbolic constants for improved readability */
 	 private final int UP = 0;
    private final int DOWN = 1;
    private final int LEFT = 2;
	 private final int RIGHT = 3;
	
	/**
	Snake initialisation
	@param startX initial X-value of the snake
	@param startY initial Y-value of the snake
	*/
	 public Snake(int startX, int startY) {
        snakeX = startX;
        snakeY = startY;
        reset();
	}
	
	/** 
	Reset snake with initial parameters
	*/
	public void reset() {
        direction = 4;
        snakeTail = new Vector(); 
        snakeTail.addElement(new Point(snakeX, snakeY));
        isDead = false;
	}

	/**
	Draw the snake with colour set to its status
	@param g the graphics object
	*/
	public void draw(Graphics g) {
        final int BODY_SIZE = 10;

        Point head;
        int i = 0;
        if (isDead())
            g.setColor(Color.red);
        else
            g.setColor(Color.blue);
       
        while (i < snakeTail.size()) {
            head = (Point)snakeTail.elementAt(i);
            g.fillRect(head.x, head.y, BODY_SIZE, BODY_SIZE);
            i++;
        }
	}

	/**
	Move the snake by changing X or Y values
	Add new position of head into the Vector
	*/
	public void move() {
        if (!isDead()) {
            if (direction == UP)
                snakeY -= 10;
            else if (direction == DOWN)
                snakeY += 10;
            else if (direction == LEFT)
                snakeX -= 10;
            else if (direction == RIGHT)
                snakeX += 10;
            if (direction != 4)  { 
                if (snakeTail.contains(new Point(snakeX, snakeY)))
                    isDead = true;
                else
                    snakeTail.addElement(new Point(snakeX, snakeY));
            }
        }
	}

	/**
	Return the status of the snake
	*/
	public boolean isDead() {
	  	 return isDead;
   }
	
	/**
	Called when direction needs change
	by user pressing respective direction key
	*/
	public void faceUp() {
        if (direction != DOWN)
            direction = UP;
  	}

	/**
	Called when direction needs change
	by user pressing respective direction key
	*/
	public void faceDown() {
        if (direction != UP)
            direction = DOWN;
  	}

	/**
	Called when direction needs change
	by user pressing respective direction key
	*/
	public void faceLeft() {
	    if (direction != RIGHT)
            direction = LEFT;
  	}

	/**
	Called when direction needs change
	by user pressing respective direction key
	*/
	public void faceRight() {
	    if (direction != LEFT)
            direction = RIGHT;
  	}
}